import logging
import socket
import errno
import sys
import os
import os.path
import importlib

from amqpservices import amqplib_ext


log = logging.getLogger('amqpservices')

DEFAULT_CONFIG_MODULE_NAME = 'amqpservices_config'


class ConfigData(object):

    def __init__(self):
        module_from_env = os.getenv('AMQPSERVICES_CONFIG_MODULE')
        if module_from_env:
            log.info('Importing config module from env. variable AMQPSERVICES_CONFIG_MODULE: %s',
                    module_from_env)
            module = module_from_env
        else:
            log.info('Importing default config module %s', DEFAULT_CONFIG_MODULE_NAME)
            module = DEFAULT_CONFIG_MODULE_NAME
        try:
            self.module = importlib.import_module(module)
            self._collect_queues()
        except ImportError:
            msg = 'Cannot import amqpservices config module, using empty config'
            sys.stderr.write(msg + '\n')
            log.critical(msg)
            self.module = object()
            self.queues = {}

    def _collect_queues(self):
        self.queues = {}
        for service in self.module.SERVICES:
            self.queues.update(self.module.SERVICES[service]['queues'])
            self.queues['broadcast.'+service] = {'durable': False, 'want_reply': False}

CONFIG = ConfigData()

try:
    import raven
    sentry = raven.Client(dsn=getattr(CONFIG.module, 'SENTRY_DSN', ''))
    log.info('Using sentry for error notification')
except ImportError:
    class _SentryStub(object):
        def captureMessage(self, msg): pass
        def captureException(self): pass
    sentry = _SentryStub()
    log.info('Not using sentry for error notification - no module named "raven"')


LOCALHOST_CONNECTION = dict(host='localhost', userid='guest', password='guest', ssl=False)

class AMQPBase(object):
    def init_connection_and_channel(self, connparams, connect_timeout):
        log.info('Configured SERVICES=%s', CONFIG.module.SERVICES)
        log.info('All queues=%s', CONFIG.queues)
        try:
            self.conn = amqplib_ext.Connection(host=connparams['host'], userid=connparams['userid'],
                    password=connparams['password'], ssl=connparams['ssl'],
                        use_threading=False, connect_timeout=connect_timeout)
            self.conn.transport.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            self.ch = self.conn.channel()
            self.ch.access_request('/data', active=True, write=True)
            self.ch.basic_qos(prefetch_size=0, prefetch_count=1, a_global=False)
        except:
            log.exception('While initializing broker connection')
            sentry.captureException()
            raise
    def _process_events(self, timeout=None):
        while True:
            try:
                self.conn.drain_events(timeout)
                return
            except socket.error, val:
                if val.errno == errno.EINTR:
                    print('Restarting due to interrupting by a signal')
                    pass
                else:
                    raise


